@@ -37,13 +37,6 @@ BEGIN {
 #===============================================================================
 #==== Below are all the signal handlers supplied by the programmer          ====
 #===============================================================================
-sub Skeleton_Handler {
-	my ($class, $data) = @_;
-    my $me = __PACKAGE__."->Skeleton_Handler";
-    # REPLACE the line below with the actions to be taken when $me is called
-    __PACKAGE__->show_skeleton_message($me, \@ARG, __PACKAGE__, 'pixmaps/Logo.xpm');
-}
-
 sub on_New_activate {
 	my ($class, $data) = @_;
     my $me = __PACKAGE__."->on_New_activate";
@@ -63,7 +56,7 @@ sub on_Open_activate {
 sub on_Print_activate {
 	my ($class, $data) = @_;
     my $me = __PACKAGE__."->on_Print_activate";
-    __PACKAGE__->show_skeleton_message($me, \@ARG, __PACKAGE__, 'pixmaps/Logo.xpm');
+    __PACKAGE__->show_skeleton_message($me, \@ARG, __PACKAGE__, 'pixmaps/glade2perl_logo.xpm');
 }
 
 sub on_BusFrame_delete_event {shift->destroy;Gtk->main_quit}
